// ==========================================================================
// desc: Home Article
// company:云南奇讯科技有限公司
// ==========================================================================

package logic

import (
	"context"
	"database/sql"
	"github.com/gogf/gf/v2/database/gdb"
	"github.com/gogf/gf/v2/encoding/gjson"
	"github.com/gogf/gf/v2/frame/g"
	"github.com/gogf/gf/v2/os/gtime"
	"github.com/gogf/gf/v2/text/gregex"
	"github.com/gogf/gf/v2/util/gconv"
	"github.com/tiger1103/gfast/v3/api/v1/cms"
	"github.com/tiger1103/gfast/v3/internal/app/cms/dao"
	"github.com/tiger1103/gfast/v3/internal/app/cms/model"
	"github.com/tiger1103/gfast/v3/internal/app/cms/model/do"
	"github.com/tiger1103/gfast/v3/internal/app/cms/service"
	commonService "github.com/tiger1103/gfast/v3/internal/app/common/service"
	"github.com/tiger1103/gfast/v3/internal/app/system/consts"
	systemService "github.com/tiger1103/gfast/v3/internal/app/system/service"
	"github.com/tiger1103/gfast/v3/library/liberr"
	"strings"
)

func init() {
	service.RegisterCmsArticle(New())
}

func New() *sCmsArticle {
	return &sCmsArticle{}
}

type sCmsArticle struct{}

func (s *sCmsArticle) List(ctx context.Context, req *cms.CmsArticleSearchReq) (listRes *cms.CmsArticleSearchRes, err error) {
	listRes = new(cms.CmsArticleSearchRes)
	err = g.Try(ctx, func(ctx context.Context) {
		m := dao.CmsArticle.Ctx(ctx).WithAll()
		if req.Title != "" {
			m = m.Where(dao.CmsArticle.Columns().Title+" = ?", req.Title)
		}
		if req.CategoryId != "" {
			//先查找子分类
			cateIds, err2 := service.CmsCategory().GetChildrenIds(ctx, []uint{gconv.Uint(req.CategoryId)})
			liberr.ErrIsNil(ctx, err2, "获取子分类失败")
			m = m.Where(dao.CmsArticle.Columns().CategoryId+" in(?)", cateIds)
		}
		if req.ModuleId != "" {
			m = m.Where(dao.CmsArticle.Columns().ModuleId+" = ?", gconv.Uint(req.ModuleId))
		}
		if req.Flag != "" {
			m = m.Where(" find_in_set(?,`"+dao.CmsArticle.Columns().Flag+"`)", req.Flag)
		}
		if req.Status != "" {
			m = m.Where(dao.CmsArticle.Columns().Status+" = ?", gconv.Bool(req.Status))
		}
		if len(req.CateIds) > 0 {
			m = m.Where(dao.CmsArticle.Columns().CategoryId+" in(?)", req.CateIds)
		}
		listRes.Total, err = m.Count()
		liberr.ErrIsNil(ctx, err, "获取总行数失败")
		if req.PageNum == 0 {
			req.PageNum = 1
		}
		listRes.CurrentPage = req.PageNum
		if req.PageSize == 0 {
			req.PageSize = consts.PageSize
		}
		order := "published_at DESC,id DESC"
		if req.OrderBy != "" {
			order = req.OrderBy
		}
		var res []*model.CmsArticleInfoRes
		err = m.Fields(cms.CmsArticleSearchRes{}).WithAll().Page(req.PageNum, req.PageSize).Order(order).Scan(&res)
		liberr.ErrIsNil(ctx, err, "获取数据失败")
		listRes.List = make([]*model.CmsArticleListRes, len(res))
		// 获取文章属性
		cmsArticleAttrs, _ := commonService.SysDictData().GetDictWithDataByType(ctx, "cms_article_attr", "")
		for k, v := range res {
			if v.Flag != "" {
				flagArr := strings.Split(v.Flag, ",")
				for _, flag := range flagArr {
					for _, attr := range cmsArticleAttrs.Values {
						if flag == attr.DictValue {
							v.Attr = append(v.Attr, attr.DictLabel)
						}
					}
				}
			}
			listRes.List[k] = &model.CmsArticleListRes{
				Id:              v.Id,
				Title:           v.Title,
				CategoryId:      v.CategoryId,
				ModuleId:        v.ModuleId,
				Hits:            v.Hits,
				Flag:            v.Flag,
				CommentCount:    v.CommentCount,
				FavoriteCount:   v.FavoriteCount,
				LikeCount:       v.LikeCount,
				Status:          v.Status,
				UserId:          v.UserId,
				Summary:         v.Summary,
				Attr:            v.Attr,
				Thumb:           v.Thumb,
				ReadPermissions: v.ReadPermissions,
				OriginTitle:     v.OriginTitle,
				CreatedAt:       v.CreatedAt,
				Category:        v.Category,
				UserInfo:        v.UserInfo,
			}
		}
	})
	return
}

func (s *sCmsArticle) GetById(ctx context.Context, id uint64) (res *model.CmsArticleInfoRes, err error) {
	err = g.Try(ctx, func(ctx context.Context) {
		err = dao.CmsArticle.Ctx(ctx).WithAll().Where(dao.CmsArticle.Columns().Id, id).Scan(&res)
		liberr.ErrIsNil(ctx, err, "获取信息失败")
		articleDataRes, err2 := dao.CmsArticleData.Ctx(ctx).One("aid=?", id)
		liberr.ErrIsNil(ctx, err2, "获取信息详情内容失败")
		if articleDataRes != nil || !articleDataRes.IsEmpty() {
			res.Content = articleDataRes["content"].String()
		}
		// 获取模型内容
		moduleInfo, err := systemService.ModulesInfo().GetById(ctx, res.ModuleId)
		liberr.ErrIsNil(ctx, err)
		if moduleInfo != nil {
			modulesData, _ := systemService.ModulesData().Get(ctx, moduleInfo.TableName, dao.CmsArticle.Table(), res.Id)
			modulesDataMap := modulesData.Map()
			for key, value := range modulesDataMap {
				str := gconv.String(value)
				if gregex.IsMatchString(`^\[\{|^\[`, str) {
					modulesDataMap[key], _ = gjson.Decode([]byte(str))
				}
			}
			res.ModulesData = modulesDataMap
		}
	})
	return
}

func (s *sCmsArticle) Add(ctx context.Context, req *cms.CmsArticleAddReq) (err error) {
	flag := ""
	req.Flag.FilterEmpty()
	if !req.Flag.IsEmpty() {
		flag = req.Flag.Join(",")
	}
	if req.PublishedAt == nil {
		req.PublishedAt = gtime.Now()
	}
	err = g.DB().Transaction(ctx, func(ctx context.Context, tx gdb.TX) error {
		err = g.Try(ctx, func(ctx context.Context) {
			var result sql.Result
			result, err = dao.CmsArticle.Ctx(ctx).Insert(do.CmsArticle{
				Title:           req.Title,
				Summary:         req.Summary,
				SeoTitle:        req.SeoTitle,
				CategoryId:      req.CategoryId,
				ModuleId:        req.ModuleId,
				Keywords:        req.Keywords,
				Description:     req.Description,
				Flag:            flag,
				Thumb:           req.Thumb,
				Template:        req.Template,
				UserId:          req.UserId,
				IsJump:          req.IsJump,
				JumpUrl:         req.JumpUrl,
				Status:          req.Status,
				ReadPermissions: req.ReadPermissions,
				OriginUrl:       req.OriginUrl,
				OriginTitle:     req.OriginTitle,
				PublishedAt:     req.PublishedAt,
			})
			liberr.ErrIsNil(ctx, err)
			id, _ := result.LastInsertId()
			// 添加内容表
			_, err = dao.CmsArticleData.Ctx(ctx).Insert(do.CmsArticleData{
				Aid:     id,
				Content: req.Content,
			})
			liberr.ErrIsNil(ctx, err)
			// 设置文章标签
			err = s.setArticleTags(ctx, id, req.Tags)
			liberr.ErrIsNil(ctx, err)
			// 添加模型表内容
			if req.ModulesData == nil {
				req.ModulesData = g.Map{}
			}
			req.ModulesData["relation_id"] = id
			req.ModulesData["relation_table"] = dao.CmsArticle.Table()
			moduleInfo, err := systemService.ModulesInfo().GetById(ctx, req.ModuleId)
			liberr.ErrIsNil(ctx, err)
			if moduleInfo != nil {
				err = systemService.ModulesData().Add(ctx, moduleInfo.TableName, req.ModulesData)
				liberr.ErrIsNil(ctx, err)
			}
		})
		return err
	})
	return
}

func (s *sCmsArticle) setArticleTags(ctx context.Context, articleId interface{}, tagsId []uint) (err error) {
	//删除原有标签
	_, err = dao.CmsArticleTags.Ctx(ctx).Where(dao.CmsArticleTags.Columns().ArticleId, articleId).Delete()
	if err != nil {
		return
	}
	if len(tagsId) == 0 {
		return
	}
	data := make([]do.CmsArticleTags, len(tagsId))
	for k, v := range tagsId {
		data[k] = do.CmsArticleTags{
			ArticleId: articleId,
			TagsId:    v,
		}
	}
	_, err = dao.CmsArticleTags.Ctx(ctx).Data(data).Insert()
	return
}

func (s *sCmsArticle) Edit(ctx context.Context, req *cms.CmsArticleEditReq) (err error) {
	flag := ""
	req.Flag.FilterEmpty()
	if !req.Flag.IsEmpty() {
		flag = req.Flag.Join(",")
	}
	var oldArticle *model.CmsArticleInfoRes
	err = dao.CmsArticle.Ctx(ctx).WherePri(req.Id).Scan(&oldArticle)
	liberr.ErrIsNil(ctx, err)
	err = g.DB().Transaction(ctx, func(ctx context.Context, tx gdb.TX) error {
		err = g.Try(ctx, func(ctx context.Context) {
			if req.PublishedAt == nil {
				req.PublishedAt = gtime.Now()
			}
			_, err = dao.CmsArticle.Ctx(ctx).WherePri(req.Id).Update(do.CmsArticle{
				Title:           req.Title,
				Summary:         req.Summary,
				SeoTitle:        req.SeoTitle,
				CategoryId:      req.CategoryId,
				ModuleId:        req.ModuleId,
				Keywords:        req.Keywords,
				Description:     req.Description,
				Flag:            flag,
				IsJump:          req.IsJump,
				JumpUrl:         req.JumpUrl,
				Thumb:           req.Thumb,
				Template:        req.Template,
				Status:          req.Status,
				ReadPermissions: req.ReadPermissions,
				OriginUrl:       req.OriginUrl,
				OriginTitle:     req.OriginTitle,
				PublishedAt:     req.PublishedAt,
			})
			liberr.ErrIsNil(ctx, err)
			// 更新内容表
			_, err = dao.CmsArticleData.Ctx(ctx).Where(dao.CmsArticleData.Columns().Aid+"=?", req.Id).Update(g.Map{
				"content": req.Content,
			})
			liberr.ErrIsNil(ctx, err)
			// 更新标签数据
			err = s.setArticleTags(ctx, req.Id, req.Tags)
			liberr.ErrIsNil(ctx, err)
			// 更新模型表内容
			// 这里要检查是否改了模型类型
			if oldArticle.ModuleId != req.ModuleId {
				// 删除旧模型的数据
				oldModuleInfo, _ := systemService.ModulesInfo().GetById(ctx, oldArticle.ModuleId)
				err = systemService.ModulesData().Delete(ctx, oldModuleInfo.TableName, dao.CmsArticle.Table(), oldArticle.Id)
				liberr.ErrIsNil(ctx, err)
			}
			if req.ModuleId > 0 && req.ModulesData != nil {
				req.ModulesData["relation_id"] = req.Id
				req.ModulesData["relation_table"] = dao.CmsArticle.Table()
				moduleInfo, _ := systemService.ModulesInfo().GetById(ctx, req.ModuleId)
				if moduleInfo != nil {
					err = systemService.ModulesData().Edit(ctx, moduleInfo.TableName, req.ModulesData)
					liberr.ErrIsNil(ctx, err)
				}
			}
		})
		return err
	})
	return
}

func (s *sCmsArticle) Delete(ctx context.Context, ids []uint64) (err error) {
	err = g.Try(ctx, func(ctx context.Context) {
		err = g.DB().Transaction(ctx, func(ctx context.Context, tx gdb.TX) error {
			// 删除模型内容
			for _, id := range ids {
				articleInfo, err2 := s.GetById(ctx, id)
				if err2 != nil {
					return err2
				}
				if articleInfo.ModuleId > 0 {
					moduleInfo, err2 := systemService.ModulesInfo().GetById(ctx, articleInfo.ModuleId)
					err2 = systemService.ModulesData().Delete(ctx, moduleInfo.TableName, dao.CmsArticle.Table(), id)
					if err2 != nil {
						return err2
					}
				}
			}
			_, err = dao.CmsArticle.Ctx(ctx).Delete(dao.CmsArticle.Columns().Id+" in (?)", ids)
			if err != nil {
				return err
			}
			// 删除内容
			_, err = dao.CmsArticleData.Ctx(ctx).Delete(dao.CmsArticleData.Columns().Aid+" in (?)", ids)
			if err != nil {
				return err
			}
			// 删除评论
			_, err = dao.CmsComment.Ctx(ctx).Delete(dao.CmsComment.Columns().ArticleId+" in (?)", ids)
			if err != nil {
				return err
			}
			// 删除收藏
			_, err = dao.CmsFavorite.Ctx(ctx).Delete(dao.CmsFavorite.Columns().ArticleId+" in (?)", ids)
			if err != nil {
				return err
			}
			// 删除点赞
			_, err = dao.CmsLike.Ctx(ctx).Delete(dao.CmsLike.Columns().ArticleId+" in (?)", ids)
			if err != nil {
				return err
			}
			return err
		})
		if err != nil {
			liberr.ErrIsNil(ctx, err, "删除失败"+err.Error())
		}
	})
	return
}
